#include "plist.h"
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * Written Functions                                                         *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


/* Adds a new point to the mesh.  Only new_point->price[] should already
 * be set.  Sets all fields in *new_point to either their final values
 * or the initial estimates of them.  Also updates any g_partial's in 
 * the mesh which are affected by the addition.  *parent is the point
 * from which the new point was generated.  *parent->children will be
 * incremented.
 */
void add_point_to_mesh(POINT *new_point, POINT *parent, int timeslice);


/* Calculate the mesh estimate of the exercise_value of the option given
 * the initial state described by *point.  *point must already be in the mesh
 * (see add_point_to_mesh() ).  Points may be added to the mesh to get desired
 * accuracy.  value() will be recursively called for any points added.
 */
void value(POINT *point, int timeslice);


/* Prints a string to stdout and if Trace != 0 also prints to Debug_file.
 * Use sprintf() to generate your string.
 */
void our_printf(char *string);


/* Constructs price vector final_price[] by growing each underlier price
 * in price vector initial_price[] at a random rate for one timeslice.  The
 * random growth rates are selected from the probability distribution 
 * described by Mean[] and Covariance[][].
 */
void grow_point(double *initial_price, double *final_price);


/* Calculates the payoff of the option based on the price vector
 * point->price[].  The result is deflated at Interest_rate for a time
 * point->date.
 */
double payoff(POINT *point, int timeslice);


/* The probability that if the underliers have a price vector of from->price[]
 * at one timeslice that they will have a price to->price[] at the next
 * timeslice.
 */
double transition_probability(POINT *from, POINT *to);


/* Uses the list of important_points to make a mesh_estimate of the
 * value of the option.  Returns error estimate in *error.
 */
double mesh_estimate_value(POINT *point, int timeslice, double *error);

void handle_command_line(int argc, char *argv[]);
void init_rng(void);
int main(int argc, char *argv[]);





/* Determine if POINT *to is relevant in calculating mesh estimates for
 * POINT *from based on POINT *to's weight for POINT *from.  Returns
 * 1 if relevant, 0 if not relevant.
 */
int point_is_relevant(POINT *from, POINT *to);


/* Calculates weights from point to all points at timeslice+1.
 * Any points which are determined to be important are added to 
 * the point->important_points list.  Importance is determined by
 * is_point_relevant().
 */
void find_important_points(POINT *point, int timeslice);



/* Returns 1 if the current error (*point->eror) in the estimate of 
 * value the value if held (point->value_if_held) is too large.  Returns
 * 0 otherwise.
 */
int error_too_large(POINT *point, int timeslice);



/* Intializes the first path from time 0 to final time. This puts in one
   point at each time. 
*/
int initialize_path();

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *                                                                           *
 * Unwritten Functions                                                       *
 *                                                                           *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
